<?php
class ControllerExtensionPaymentCypix extends Controller {
    
  const API_VERSION = 1;

  public function index() {
    
    $this->language->load('extension/payment/cypix');
    $this->load->model('checkout/order');
    
    $token = $this->generateToken();
    
    $data['action'] = $token;
    $data['button_confirm'] = $this->language->get('button_confirm');
    $data['token'] = $token;
    $data['token_error'] = $this->language->get('token_error');
    $data['order_id'] = $this->session->data['order_id'];

    return $this->load->view('extension/payment/cypix', $data);
  }

  public function generateToken(){

    $this->load->model('checkout/order');
    
    $order_info = $this->model_checkout_order->getOrder($this->session->data['order_id']);
    
    $orderAmount = $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value'], false);
    
    $orderAmount = (float)$orderAmount * pow(10,(int)$this->currency->getDecimalPlace($order_info['currency_code']));
    
    $orderAmount = intval(strval($orderAmount));
    
    $salt = mt_rand();
    
    $params =   [
                    'amount'             => $order_info['total'],
                    'currency'           => $order_info['currency_code'],
                    'description'        => $this->language->get('text_order'). ' ' .$order_info['order_id'],
                    'returnUrl'          => $this->url->link('extension/payment/cypix/callback&', '', 'SSL'),
                    'notificationUrl'    => $this->url->link('extension/payment/cypix/callback1', '', 'SSL'),
                    'method'             => 'card',
                    'orderId'            => $order_info['order_id'],
                    'customer'           => [
                                                'email'     => strlen($order_info['email']) > 0 ? $order_info['email'] : null,
                                                'language'  => $this->_language($this->session->data['language']),
                                                'country'   => strlen($order_info['payment_iso_code_2']) > 0 ? $order_info['payment_iso_code_2'] : null
                                            ]
                ];
                    
    $postdata = json_encode($params);
        
    $signature = $this -> getSignature($postdata, $salt, $this->config->get('payment_cypix_encyptionkey'));
     
    $headers = [
                       "Content-Type: application/json",
                       "X-Signature: ".$signature,
                       "X-Service-Api-Key: ".$this->config->get('payment_cypix_companyid'),
                       "X-Salt: ".$salt  
               ];
         
    $options = [
                       CURLOPT_HTTPHEADER     => $headers,
			           CURLOPT_POSTFIELDS     => $postdata,
               ];
                    
    $result = $this -> makeRequest( $this->config->get('payment_cypix_gateway_url').'gateway/form', $options );
        
    $data = json_decode($result, true);
    
    if ($data && isset($data['status']) && $data['status'] == 'ACCEPTED') {
        
          return $data['form']['url'];
            
    } else {
        
          $this->log->write("Payment token response parse error: ".$data['status']);
          
          return false;  
    }
  }

  public function callback() {

    if (isset($this->session->data['order_id'])) {
      $order_id = $this->session->data['order_id'];
    } else {
      $order_id = 0;
    }

    $this->load->model('checkout/order');
    $order_info = $this->model_checkout_order->getOrder($order_id);

    $this->response->redirect($this->url->link('checkout/success', '', 'SSL'));
  }

  public function callback1() {
    
    $raw_post = file_get_contents( 'php://input' );
        
    $decoded  = json_decode( $raw_post, true );
    
    $this->log->write("Webhook received: $postData");

    $this->load->model('checkout/order');
    
	if ( isset( $decoded['transaction'] ) ) {
		  
		          if (!isset($decoded['transaction']['orderId'])) {
			             $this->log->write("Payment is failed");
		          }
                  
                  $order_id = $decoded['transaction']['orderId'] ? $decoded['transaction']['orderId'] : false;
                  
                  $order_info = $this->model_checkout_order->getOrder($order_id);

                  if ($order_info) {
                        $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('config_order_status_id'));

                        if(isset($decoded['transaction']['status']) && $decoded['transaction']['status'] == 'PROCESSED'){
                                    $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_cypix_completed_status_id'), "UID: $order_id", true);
                                    
                        } else {
                                $this->model_checkout_order->addOrderHistory($order_id, $this->config->get('payment_cypix_failed_status_id'), "UID: $order_id", true);
                        }
                        
                  } else {
                    
                        $this->log->write("Payment is failed");
                  }
	} else {
	   $this->log->write("Payment is failed");
	}
  }

  private function _language($lang_id) {
    $lang = substr($lang_id, 0, 2);
    $lang = strtolower($lang);
    return $lang;
  }
  
  public function getSignature($postdata, $salt, $secret) {
  
        $signatureRaw = hash_hmac('sha1', $postdata . $salt , $secret, true);
    
        $signature = base64_encode($signatureRaw);
        
        return $signature;
  }
  
   public function makeRequest( $url, array $options = [ ] ) {
    
		$defaults = [
			CURLOPT_POST           => 1,
			CURLOPT_HEADER         => 0,
			CURLOPT_URL            => $url,
			CURLOPT_FRESH_CONNECT  => 1,
			CURLOPT_RETURNTRANSFER => 1,
			CURLOPT_FORBID_REUSE   => 1,
			CURLOPT_TIMEOUT        => 4,
			CURLOPT_SSL_VERIFYHOST => 2,
			CURLOPT_SSL_VERIFYPEER => false
		];

		$ch = curl_init();
		curl_setopt_array( $ch, ( $options + $defaults ) );
        
        $result = curl_exec( $ch );

		if (!$result) {
			throw new Exception( curl_error( $ch ) );
		}
		curl_close( $ch );

		return $result;
  }
}
